import json

import requests

from stayfulrestsdk.exceptions import APIException

# DEFAULT_API_URL = 'http://localhost:8080/v1'
DEFAULT_API_URL = 'https://api.sandbox.stayful.com/v1'
# DEFAULT_API_URL = 'https://api.stayful.com/v1'

# global variables used by get_default_api()
GLOBAL_API_KEY = None
GLOBAL_API_URL = DEFAULT_API_URL
GLOBAL_API_CLIENT = None


class API(object):
    METHODS = {
        'get': requests.get,
        'post': requests.post,
        'patch': requests.patch,
        'delete': requests.delete,
        'put': requests.put,
    }

    def __init__(self, api_key, url=DEFAULT_API_URL):
        self.api_key = api_key
        self.url = url

    def build_endpoint(self, endpoint):
        if endpoint.startswith('/'):
            endpoint = endpoint[1:]
        return '{}/{}'.format(self.url, endpoint)

    def request(self, method, url, params=None, data=None):
        if not url.startswith('http'):
            url = self.build_endpoint(url)

        method = self.METHODS[method.lower()]

        auth = (self.api_key, None)
        headers = {
            'content-type': 'application/json',
            'accept': 'application/json',
        }

        if data:
            data = json.dumps(data)

        response = method(url, auth=auth, headers=headers, params=params, data=data)

        if not (200 <= response.status_code < 300):
            try:
                msg = response.json()['message']
                exc = APIException(msg)
            except:
                exc = APIException('Unknown error. Response body: {}'.format(response.content))

            raise exc

        return response


def configure(api_key, url=DEFAULT_API_URL):
    global GLOBAL_API_KEY, GLOBAL_API_URL
    GLOBAL_API_KEY = api_key
    GLOBAL_API_URL = url


def get_default_api():
    global GLOBAL_API_CLIENT
    if GLOBAL_API_CLIENT is None:
        if GLOBAL_API_KEY is None:
            raise Exception('api_key is not set, use configure() to set it.')
        GLOBAL_API_CLIENT = API(api_key=GLOBAL_API_KEY, url=GLOBAL_API_URL)
    return GLOBAL_API_CLIENT

    return API(api_key=API_KEY, url=API_URL)

