from stayfulrestsdk.resource import Resource, DeleteMixin, GetMixin, CreateMixin
from stayfulrestsdk.api import get_default_api
from stayfulrestsdk.exceptions import APIException

class Card(DeleteMixin, Resource):
    pass


class Rate(Resource):
    path = '/rates'

    @classmethod
    def search(cls, hotels, checkin, checkout, api=None):
        api = api or get_default_api()
        hotel_ids = [hotel.id for hotel in hotels]
        params = {
            'hotel_id': hotel_ids,
            'checkin': checkin,
            'checkout': checkout,
        }
        resp = api.request('get', cls.path, params=params)
        return [cls(item, api) for item in resp.json()]



class Hotel(GetMixin, Resource):
    path = '/hotels'
    
    @classmethod
    def search(cls, params=None, api=None):
        # convert python booleans to required format
        if params and 'contracted' in params:
            params['contracted'] = 'true' if params['contracted'] else 'false'

        api = api or get_default_api()
        response = api.request('get', cls.path, params)
        return [cls(item, api) for item in response.json()]


class Bid(GetMixin, CreateMixin, Resource):
    path = '/bids'
    
    def get_url(self):
        if 'id' in self.attrs:
            return '{path}/{id}'.format(path=self.path, id=self.attrs['id'])

    @property
    def payment_url(self):
        if 'payment_url' in self.attrs:
            return self.attrs['payment_url']
        else:
            return self.url + '/payment'

    def pay(self, card, first_name, last_name):
        data = {
            'card_id': card.card_id,
            'guest_details': {
                'first_name': first_name,
                'last_name': last_name,
            }
        }
        resp = self.api.request('post', self.payment_url, data=data)
        self.merge(resp.json())


class Customer(CreateMixin, GetMixin, Resource):
    path = '/customers'
    embed = {'cards': Card, 'bids': Bid}

    def get_url(self):
        if 'id' in self.attrs:
            return '{path}/{id}'.format(path=self.path, id=self.attrs['id'])

    @property
    def cards_url(self):
        if 'cards_url' in self.attrs:
            return self.attrs['cards_url']
        else:
            return self.url + '/cards'

    def add_card(self, card):
        resp = self.api.request('post', self.cards_url, data=card.attrs)
        card.merge(resp.json())
        self.attrs.setdefault('cards', []).append(card)
        return card